# Copyright (C) 2018 NTT DATA
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import itertools

from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from horizon.utils import functions as utils
from horizon.utils import memoized
from keystoneauth1.identity.generic import token
from keystoneauth1 import session as ks_session
from openstack import connection

from masakaridashboard.handle_errors import handle_errors


@memoized.memoized
def openstack_connection(request):
    auth = token.Token(
        auth_url=getattr(settings, 'OPENSTACK_KEYSTONE_URL'),
        token=request.user.token.id,
        project_name=request.user.project_name,
        project_id=request.user.tenant_id)
    session = ks_session.Session(auth=auth)
    conn = connection.Connection(session=session)

    return conn.instance_ha


@handle_errors(_("Unable to retrieve segments"), [])
def get_segment_list(request, marker='', paginate=False, filters=None):
    """Returns segments as per page size."""
    page_size = utils.get_page_size(request)
    client = openstack_connection(request)
    kwargs = get_request_param(marker, paginate, filters, page_size)
    entities_iter = client.segments(**kwargs)
    has_prev_data = has_more_data = False
    if paginate:
        entities, has_more_data, has_prev_data = pagination_process(
            entities_iter, kwargs['limit'], page_size, marker)
    else:
        entities = list(entities_iter)

    return entities, has_more_data, has_prev_data


def get_request_param(marker, paginate, filters, page_size):
    limit = getattr(settings, 'API_RESULT_LIMIT', 100)

    if paginate:
        request_size = page_size + 1
    else:
        request_size = limit

    kwargs = {"marker": marker,
              "limit": request_size
              }
    if filters is not None:
        kwargs.update(filters)
    return kwargs


def pagination_process(data, request_size, page_size, marker):
    """Retrieve a listing of specific entity and handles pagination.

    :param request: Request data
    :param marker: Pagination marker for large data sets: entity id
    :param paginate: If true will perform pagination based on settings.
                     Default:False
    """
    prev_data = more_data = False
    entities = list(itertools.islice(data, request_size))
    # first and middle page condition
    if len(entities) > page_size:
        entities.pop()
        more_data = True
        # middle page condition
        if marker is not None:
            prev_data = True
    elif marker is not None:
        prev_data = True

    return entities, more_data, prev_data


@handle_errors(_("Unable to retrieve segments"), [])
def segment_list(request):
    return list(openstack_connection(request).segments())


def segment_create(request, data):
    """Create segment."""
    return openstack_connection(request).create_segment(**data)


@handle_errors(_("Unable to retrieve segment"), [])
def get_segment(request, segment_id):
    """Returns segment by id"""
    return openstack_connection(request).get_segment(segment_id)


@handle_errors(_("Unable to delete segment"), [])
def segment_delete(request, segment_id, ignore_missing=True):
    return openstack_connection(request).delete_segment(
        segment_id, ignore_missing)


@handle_errors(_("Unable to update segment"), [])
def segment_update(request, segment_id, fields_to_update):
    """Update segment."""
    return openstack_connection(request).update_segment(
        segment_id, **fields_to_update)



# Copyright (C) 2018 NTT DATA
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.conf import settings
from django.core.urlresolvers import reverse
from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from horizon import tables
from masakaridashboard.api import api
from masakaridashboard.segments import tables as masakari_tab

from horizon import exceptions
from horizon import forms

from horizon.utils import memoized
from masakaridashboard.segments import forms as segment_forms

from horizon import tabs
from masakaridashboard.segments import tabs as seg_tab


class IndexView(tables.DataTableView):
    table_class = masakari_tab.FailoverSegmentTable
    template_name = 'masakaridashboard/segments/index.html'
    page_title = _("Segments")

    _more = False
    _prev = False

    def needs_filter_first(self, table):
        return self._needs_filter_first

    def has_more_data(self, table):
        return self._more

    def has_prev_data(self, table):
        return self._prev

    def get_data(self):
        segments = []
        marker = self.request.GET.get(
            masakari_tab.FailoverSegmentTable._meta.pagination_param,
            None
        )
        if marker is not None:
            segment = api.get_segment(self.request, marker)
            marker = segment.id
        filters = self.get_filters()
        self._needs_filter_first = True

        filter_first = getattr(settings, 'FILTER_DATA_FIRST', {})
        if filter_first.get('masakaridashboard.segments', False) and len(
                filters) == 0:
            self._needs_filter_first = True
            self._more = False
            return segments

        try:
            segments, self._more, self._prev = api.get_segment_list(
                request=self.request,
                marker=marker,
                filters=filters,
                paginate=True
            )
        except Exception:
            self._prev = False
            self._more = False
            msg = _('Unable to retrieve segment list.')
            exceptions.handle(self.request, msg)

        return segments


class CreateSegmentView(forms.ModalFormView):
    template_name = 'masakaridashboard/segments/create.html'
    modal_header = _("Create Segment")
    form_id = "create_segment"
    form_class = segment_forms.CreateSegmentForm
    submit_label = _("Create")
    submit_url = reverse_lazy(
        "horizon:masakaridashboard:segments:create_segment")
    success_url = reverse_lazy("horizon:masakaridashboard:segments:index")
    page_title = _("Create Segment")

    def get_form_kwargs(self):
        kwargs = super(CreateSegmentView, self).get_form_kwargs()
        return kwargs


class DetailView(tabs.TabbedTableView):
    tab_group_class = seg_tab.SegmentDetailTabs
    template_name = 'horizon/common/_detail.html'
    page_title = "{{ segment.name|default:segment.id }}"

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        segment = self.get_data()
        table = masakari_tab.FailoverSegmentTable(self.request)
        context["segment"] = segment
        context["url"] = self.get_redirect_url()
        context["actions"] = table.render_row_actions(segment)
        return context

    @memoized.memoized_method
    def get_data(self):
        try:
            segment_id = self.kwargs['segment_id']
            segment = api.get_segment(self.request, segment_id)
        except Exception:
            msg = _('Unable to get segment "%s".') % segment_id
            redirect = reverse('horizon:masakaridashboard:segments:index')
            exceptions.handle(self.request, msg, redirect=redirect)

        return segment

    def get_redirect_url(self):
        return reverse('horizon:masakaridashboard:segments:index')

    def get_tabs(self, request, *args, **kwargs):
        segment = self.get_data()
        return self.tab_group_class(request, segment=segment, **kwargs)


class UpdateView(forms.ModalFormView):
    template_name = 'masakaridashboard/segments/update.html'
    modal_header = _("Update Segment")
    form_id = "update_segment"
    form_class = segment_forms.UpdateForm
    submit_label = _("Update")
    submit_url = "horizon:masakaridashboard:segments:update"
    success_url = reverse_lazy("horizon:masakaridashboard:segments:index")
    page_title = _("Update Segment")

    @memoized.memoized_method
    def get_object(self):
        try:
            segment = api.get_segment(self.request, self.kwargs['segment_id'])
            return segment
        except Exception:
            msg = _('Unable to retrieve segment.')
            redirect = reverse('horizon:masakaridashboard:segments:index')
            exceptions.handle(self.request, msg, redirect=redirect)

    def get_context_data(self, **kwargs):
        context = super(UpdateView, self).get_context_data(**kwargs)
        context['submit_url'] = reverse(
            self.submit_url,
            args=[self.kwargs["segment_id"]]
        )

        return context

    def get_initial(self, **kwargs):
        segment = self.get_object()

        return {'uuid': self.kwargs['segment_id'],
                'name': segment.name,
                'recovery_method': segment.recovery_method,
                'description': segment.description}
